#!/usr/bin/python

from pyaspell import Aspell
import sys

from cgi import FieldStorage
from traceback import format_exc


GRID_TEXT_EXAMPLE = '''
ZNTKHAKIPTX
TATTYSYTTID
FHJELTENSUE
AKEAVSSCANT
REGCTITAAEE
DKLIIERYRXS
BENKNEZTDSS
RUDENCBSSDA
UYTDHAEUPLA
LEUAASBROAL
BAPNIOAIOCT
CSQUGIVUNSA
HTRUSTHGIER
EASESICINGH
'''

GRID_TEXT = FieldStorage().getfirst('grid', None) or GRID_TEXT_EXAMPLE

def combinations( group ):
	"""
	Yields all letter groups from a letter grouping
	"""
	for offset in xrange( len( group ) ):
		fragment = group[ offset: ]
		for size in xrange( 1, len(fragment)+1 ):
			yield fragment[:size]

def list_to_word(group):
	word = ''
	for a in group:
		word = '%s%s' % (word, a)
	return word

def unique( li ):
	groups = []
	for a in map(list, li):
		if (a) and (not a in groups):
			groups.append( a )
	return groups

class Grid( object ):
	def __init__(self, grid_text):
		object.__init__(self)
		self.grid = filter( lambda x: x, grid_text.split() )
		(self.h, self.w) = self._gridsize( self.grid )
	
	def _gridsize( self, grid ):
		h = len(grid)
		w = None
		
		for y in xrange(h):
			thisw = len( grid[y] )
			if w:
				if not w == thisw:
					raise Exception('Grid size varies on line %d (reading "%s")' %(y, grid[y]) )
			else:
				w = thisw
		
		if (not h) or (not w):
			raise Exception('Invalid grid size')
		
		return (h, w)
	
	def inside( self, x, y ):
		return ( ( x >= 0 ) and ( y >= 0 ) and ( x < self.w ) and (y < self.h) )
	
	def get( self, x, y ):
		if not self.inside(x, y):
			raise Exception( 'Position (%d, %d) out of grid.' % (x, y) )
		return self.grid[y][x]
	
	def rows(self):
		for y in xrange(self.h):
			yield self.row(y)
	
	def cols(self):
		for x in xrange(self.w):
			yield self.col(x)
	
	def diags(self, leading):
		for i in xrange( -self.w, self.h ):
			yield self.diag(i, leading)
	
	def row( self, y ):
		text = []
		for x in xrange( self.w ):
			text.append( self.get(x,y) )
		return text
	
	def col( self, x ):
		text = []
		for y in xrange( self.h ):
			text.append( self.get(x,y) )
		return text
	
	def diag( self, i, leading ):
		text = []
		for x in xrange( -self.h, self.w ):
			if leading:
				y = x - i
			else:
				y = self.h - x + i
			if self.inside( x,y ):
				text.append( self.get(x,y) )
		return text
	
	def all( self ):
		for col in self.cols():
			yield col
			yield reversed( col )
		for row in self.rows():
			yield row
			yield reversed( row )
		for diagx in self.diags(True):
			yield diagx
			yield reversed( diagx )
		for diagy in self.diags(False):
			yield diagy
			yield reversed( diagy )
	
	def unique( self ):
		return unique(self.all())
	
	def __str__( self ):
		text = '<GRID::'
		for r in self.rows():
			text = '%s\n\t%s' %( text, str(r) )
		return '%s\n>' % text

if __name__=='__main__':
	try:
		print 'Content-type: text/plain\r\n\r\nGenerating word list for the following grid:\n\n'
		
		def wordsort( a, b ):
			if len(a) > len(b):
				return +1
			if len(a) < len(b):
				return -1
			if a > b:
				return +1
			if a < b:
				return -1
			return 0
		
		g = Grid(GRID_TEXT)
		a = Aspell(("lang", "en"))
		wordlist = []
		
		print g
		print '\n\nWords found:\n\n'
		
		possible_words = []
		for group in g.unique():
			for c in combinations( group ):
				if len(c) > 1:
					possible_words.append( c )
		for word in unique( possible_words ):
			word = list_to_word(word)
			if a.check( word.lower() ):
				wordlist.append( word )
		
		wordlist.sort( wordsort )
		print '\n'.join( wordlist )
	except Exception, err:
		print '\n\nThere is a problem:\n\n'
		print err
		#print format_exc()
